package cn.com.infosec.math.ec;

import java.math.BigInteger;

public class ECAlgorithms {
   public static ECPoint sumOfTwoMultiplies(ECPoint P, BigInteger a, ECPoint Q, BigInteger b) {
      ECCurve var4;
      if ((var4 = P.getCurve()).equals(Q.getCurve())) {
         return var4 instanceof ECCurve.F2m && ((ECCurve.F2m)var4).isKoblitz() ? P.multiply(a).add(Q.multiply(b)) : implShamirsTrick(P, a, Q, b);
      } else {
         throw new IllegalArgumentException("P and Q must be on same curve");
      }
   }

   public static ECPoint shamirsTrick(ECPoint P, BigInteger k, ECPoint Q, BigInteger l) {
      if (P.getCurve().equals(Q.getCurve())) {
         return implShamirsTrick(P, k, Q, l);
      } else {
         throw new IllegalArgumentException("P and Q must be on same curve");
      }
   }

   private static ECPoint implShamirsTrick(ECPoint P, BigInteger k, ECPoint Q, BigInteger l) {
      int var10000 = Math.max(k.bitLength(), l.bitLength());
      ECPoint var4 = P.add(Q);
      ECPoint var5 = P.getCurve().getInfinity();

      for(int var6 = var10000 - 1; var6 >= 0; --var6) {
         var5 = var5.twice();
         if (k.testBit(var6)) {
            if (l.testBit(var6)) {
               var5 = var5.add(var4);
            } else {
               var5 = var5.add(P);
            }
         } else if (l.testBit(var6)) {
            var5 = var5.add(Q);
         }
      }

      return var5;
   }
}
